from django.db import models
from django.contrib.sites.models import Site
from django.contrib.postgres.fields import JSONField, ArrayField
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string
import uuid
import json


class ThemeSettings(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    site = models.OneToOneField(Site, verbose_name=_('Site'), on_delete=models.PROTECT, related_name='settings')

    data = JSONField(_('Data'), default=dict, blank=True)

    added = models.DateTimeField(_('Added'), auto_now_add=True)
    changed = models.DateTimeField(_('Changed'), auto_now=True)

    class Meta:
        verbose_name = _('Theme settings')
        verbose_name_plural = _('Theme settings')

    def __str__(self):
        return str(self.site)

    def get_upload_path(self, filename=None):
        from .utils import get_settings_upload_path
        return get_settings_upload_path(self.site_id, filename)

    def get_settings_fields(self):
        from django.core.files.storage import default_storage
        storage = default_storage
        result = json.loads(render_to_string('pcart/settings_schema.json'))

        _current = self.data.get('current', {})

        for group in result:
            for o in group.get('settings', []):
                if 'id' in o:
                    if o.get('type') in ['radio', 'select', 'text', 'collection', 'blog', 'link_list', 'color', 'textarea']:
                        o['value'] = _current.get(o['id'], o.get('default', ''))
                    elif o.get('type') in ['image']:
                        value = _current.get(o['id'], o.get('default', ''))
                        if value:
                            value = self.get_upload_path(value)
                            o['url'] = storage.url(value)
                        o['value'] = value
                    elif o.get('type') in ['checkbox']:
                        o['value'] = _current.get(o['id'], o.get('default', False))
        return result



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-09 19:30
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='ThemeSettings',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, default=dict, verbose_name='Data')),
                ('added', models.DateTimeField(auto_now_add=True, verbose_name='Added')),
                ('changed', models.DateTimeField(auto_now=True, verbose_name='Changed')),
                ('site', models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, related_name='settings', to='sites.Site', verbose_name='Site')),
            ],
            options={
                'verbose_name': 'Theme settings',
                'verbose_name_plural': 'Theme settings',
            },
        ),
    ]



from django import forms
from .models import ThemeSettings


class EditThemeSettingsDataForm(forms.ModelForm):
    class Meta:
        model = ThemeSettings
        fields = '__all__'


class EditThemeSettingsForm(forms.ModelForm):
    class Meta:
        model = ThemeSettings
        fields = '__all__'

    def __init__(self, *args, **kwargs):
        super(EditThemeSettingsForm, self).__init__(*args, **kwargs)
        self.images_ids = []
        self.old_data = {}
        if 'instance' in kwargs:
            self.obj = kwargs['instance']
            if self.obj is not None:
                self.settings_fields = self.obj.get_settings_fields()
                self.old_data = self.obj.data
                for group in self.settings_fields:
                    for o in group['settings']:
                        if o['type'] in ['checkbox']:
                            self.fields[o['id']] = forms.BooleanField(required=False)
                        elif o['type'] in ['radio', 'select', 'text', 'collection', 'textarea', 'color', 'blog', 'link_list']:
                            self.fields[o['id']] = forms.CharField(required=False)
                        elif o['type'] in ['image']:
                            self.fields[o['id']] = forms.ImageField(required=False)
                            if self.obj.data.get(o['id']):
                                self.images_ids.append(o['id'])

    def save(self, commit=True):
        from django.core.files.uploadedfile import UploadedFile
        instance = super(EditThemeSettingsForm, self).save(commit=False)

        _old_current = self.old_data.get('current', {})
        _current = dict()

        for i in self.images_ids:
            if i in _old_current:
                _current[i] = _old_current[i]

        for k, v in self.cleaned_data.items():
            if k not in ['site', 'data']:
                if issubclass(type(v), UploadedFile):
                    _current[k] = k
                else:
                    if k not in self.images_ids:
                        _current[k] = v
                    elif v is False:
                        del _current[k]
        instance.data['current'] = _current
        if commit:
            instance.save()
        return instance




from django.contrib.sites.shortcuts import get_current_site
from .models import ThemeSettings


def pcart_settings(request):
    site = get_current_site(request)
    try:
        theme_settings = ThemeSettings.objects.get(site=site)
        _settings = theme_settings.data.get('current', {})
    except ThemeSettings.DoesNotExist:
        _settings = {}
    context = {
        'site': site,
        'settings': _settings,
    }
    return context



from distutils.core import setup
from setuptools import find_packages

INSTALL_REQUIREMENTS = [
    'Django>=1.10.5',
    'Markdown==2.6.8',
    'django-cms>=3.4.2',
]

setup(
    name='pcart-core',
    version='1.98.2',
    author='Oleh Korkh',
    author_email='oleh.korkh@the7bits.com',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    license='BSD License',
    description='A powerful e-commerce solution for Django CMS',
    long_description=open('README.txt').read(),
    platforms=['OS Independent'],
    url='http://the7bits.com/',
    install_requires=INSTALL_REQUIREMENTS,
)


from django import template
from django.utils.safestring import mark_safe
from django.conf import settings

register = template.Library()


@register.filter
def get_element(iterable, index):
    return iterable.get(index)

@register.filter
def index_element(iterable, index):
    return iterable[index]

@register.filter
def to_list(obj):
    return list(obj)

@register.filter
def sort(iterable):
    return iterable.sort()


@register.filter
def sort_by_attr(iterable, attr):
    result = sorted(iterable, key=lambda x: getattr(x, attr))
    return result


@register.filter
def sort_by_key(iterable, key):
    result = sorted(iterable, key=lambda x: x[key])
    return result


@register.filter
def widget_css_class(field, css_class):
    return field.as_widget(attrs={"class": css_class})


@register.filter
def widget_placeholder(field, value):
    return field.as_widget(attrs={"placeholder": value})


@register.simple_tag
def widget_attrs(field, **kwargs):
    return field.as_widget(attrs=kwargs)


@register.filter
def markdown(text):
    import markdown
    return mark_safe(markdown.markdown(text))


@register.filter
def subtract(value, arg):
    return value - arg


@register.filter
def add(value, arg):
    return value + arg


@register.simple_tag(takes_context=True)
def theme_url(context, filename):
    from ..utils import get_settings_upload_url
    return get_settings_upload_url(context['site'].id, filename)


@register.simple_tag
def optional_url(url_name, *args, **kwargs):
    from django.urls import reverse, NoReverseMatch
    try:
        return reverse(url_name, args=args, kwargs=kwargs)
    except NoReverseMatch:
        return '#unknown-url'

@register.filter
def is_checkbox(field):
    from django.forms import CheckboxInput
    return isinstance(field.field.widget, CheckboxInput)


@register.filter
def weight(value):
    return '%.1f kg' % value


@register.simple_tag
def create_list(*args):
    return args


@register.simple_tag
def create_dict(**kwargs):
    return kwargs


@register.simple_tag
def save(value):
    return value


@register.filter
def attrs_list(obj):
    return dir(obj)


@register.filter
def inspect(obj):
    attrs = dir(obj)
    result = dict()
    for attr in attrs:
        result[attr] = getattr(obj, attr)
    return result


@register.simple_tag
def split_str(value, delimiter=','):
    return value.split(delimiter)


@register.filter
def money(value):
    if hasattr(settings, 'PCART_CURRENCIES'):
        currency_config = settings.PCART_CURRENCIES.get(getattr(settings, 'PCART_DEFAULT_CURRENCY', 'default'))
        return currency_config.get('price_template', '%s') % value
    else:
        return value


# Querysets

@register.filter
def order_by(queryset, values):
    values_attrs = values.split(',')
    return queryset.order_by(*values_attrs)

# Liquid compatibility


@register.filter
def append(value, arg):
    """Appends characters to a string.
    Input:
        {{ 'sales' | append: '.jpg' }}
    Output:
        sales.jpg
    """
    return value + arg


@register.filter
def camelcase(value):
    """Converts a string into CamelCase.
    Input:
        {{ 'coming-soon' | camelcase }}
    Output:
        ComingSoon
    """
    chunks = value.split('-')
    return ''.join([x.title() for x in chunks])


@register.filter
def capitalize(value):
    """Capitalizes the first word in a string.
    Input:
        {{ 'capitalize me' | capitalize }}
    Output:
        Capitalize me
    """
    return value.title()


@register.filter
def downcase(value):
    """Converts a string into lowercase.
    Input
        {{ 'UPPERCASE' | downcase }}
    Output
        uppercase
    """
    return value.lower()


@register.filter
def escape(value):
    """Escapes a string.
    Input
        {{ "<p>test</p>" | escape }}
    Output
         <!-- The <p> tags are not rendered -->
        <p>test</p>"""
    import html
    return html.escape(value)


@register.filter
def upcase(value):
    return value.upper()





def get_settings_upload_path(site_id, filename):
    import os
    path = os.path.join(
        'theme-config',
        'site-%s' % site_id,
    )
    if filename is not None:
        path = os.path.join(path, filename)
    return path


def get_settings_upload_url(site_id, filename):
    from django.core.files.storage import default_storage
    path = get_settings_upload_path(site_id, filename)
    return default_storage.url(path)



from django.contrib import admin
from .models import ThemeSettings
from .forms import EditThemeSettingsForm, EditThemeSettingsDataForm


class ThemeSettingsAdmin(admin.ModelAdmin):
    list_display = ('site', 'added', 'changed')
    search_fields = ['site']

    def get_form(self, request, obj=None, **kwargs):
        if obj is not None:
            if request.GET.get('rawjson', 'no') == 'yes':
                kwargs['form'] = EditThemeSettingsDataForm
                self.exclude = None
            else:
                kwargs['form'] = EditThemeSettingsForm
                self.exclude = ('data',)
        return super(ThemeSettingsAdmin, self).get_form(request, obj, **kwargs)

    def get_readonly_fields(self, request, obj=None):
        if obj is not None:
            return ['site']
        else:
            return []

    def change_view(self, request, object_id, form_url='', extra_context=None):
        from django.core.files.storage import default_storage
        extra_context = extra_context or {}

        result = super(ThemeSettingsAdmin, self).change_view(request, object_id, form_url, extra_context=extra_context)

        if object_id and request.method == 'POST':
            _old_instance = ThemeSettings.objects.get(pk=object_id)
            for filename in request.FILES:
                f = request.FILES[filename]
                fullpath = _old_instance.get_upload_path(filename)
                if default_storage.exists(fullpath):
                    default_storage.delete(fullpath)
                default_storage.save(fullpath, f)
            for k, v in request.POST.items():
                if k.endswith('-clear'):
                    filename = k[:-len('-clear')]
                    if v == 'on':
                        fullpath = _old_instance.get_upload_path(filename)
                        if default_storage.exists(fullpath):
                            default_storage.delete(fullpath)
        return result


admin.site.register(ThemeSettings, ThemeSettingsAdmin)
